#!/usr/bin/env python3
# coding=utf-8
"""
Panasonic air conditioner classes
"""
import logging

import bitstring
import sys
from bitstring import BitArray

from eakon import HVAC
from eakon.enums import panasonic_enum


def _convert_byte_endianness(byte: str) -> str:
    assert len(byte) == 8
    byte = list(byte)
    byte.reverse()
    return "".join(byte)


def _convert_int_endianness(val: int) -> int:
    assert 0 <= val <= 255
    return int(_convert_byte_endianness("{:08b}".format(val)), 2)


class Panasonic(HVAC):
    """
    Panasonic basic remote control
    """

    __INTER_FRAME_SPACE = 10000
    __HDR_FIRST_MARK = 3500
    __HDR_FIRST_SPACE = 1750
    __MARK = 444
    __ONE_SPACE = 1300
    __ZERO_SPACE = 430
    __temp_max = 30
    __temp_min = 16
    __start_mark = [__HDR_FIRST_MARK, __HDR_FIRST_SPACE]
    __inter_frame_mark = [__MARK, __INTER_FRAME_SPACE]

    frame = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, enum=panasonic_enum)
        self.one_mark = self.__MARK
        self.one_space = self.__ONE_SPACE
        self.zero_mark = self.__MARK
        self.zero_space = self.__ZERO_SPACE

    def _get_wave(self):

        wave = self.__start_mark.copy()
        # for x in self._get_bitstring_frame1():
        #     wave.extend(self._get_zero() if x == "0" else self._get_one())
        # wave.extend(self.__inter_frame_mark)
        # wave.extend(self.__start_mark)

        for x in self._get_bitstring_frame2():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])
        return wave

    def _get_bitstring(self):
        return self._get_bitstring_frame2()

    def _get_bitstring_frame1(self):
        fmt = """
        uint: 8 = b1,
        uint: 8 = b2,
        uint: 8 = b3,
        uint: 8 = b4,
        uint: 8 = b5,
        uint: 8 = b6,
        uint: 8 = b7,
        uint: 8 = b8
        """

        frame1_data = {
            "b1": 0x02,
            "b2": 0x20,
            "b3": 0xe0,
            "b4": 0x04,
            "b5": 0x00,
            "b6": 0x00,
            "b7": 0x00,
            "b8": 0x06,
        }
        self._reverse(frame1_data)

        return bitstring.pack(fmt, **frame1_data).bin

    def _get_bitstring_frame2(self):
        fmt = """
        uint: 8 = b1,
        uint: 8 = b2,
        uint: 8 = b3,
        uint: 8 = b4,
        uint: 8 = b5,
        uint: 8 = b6,
        uint: 8 = b7,
        uint: 8 = b8,
        uint: 8 = b9,
        uint: 8 = b10,
        uint: 8 = b11,
        uint: 8 = b12,
        uint: 8 = b13,
        uint: 8 = b14,
        uint: 8 = b15,
        uint: 8 = b16,
        uint: 8 = b17,
        uint: 8 = b18,
        uint: 8 = cs
        """
        frame2_data = {
            "b1": 0x02,
            "b2": 0x20,
            "b3": 0xe0,
            "b4": 0x04,
            "b5": 0x00,
            "b6": self._get_power_status_and_mode(),
            "b7": _convert_int_endianness(2 * self.temperature),
            "b8": 0x80,
            "b9": _convert_int_endianness(self._get_fan_settings()),
            "b10": 0x00,
            "b11": 0x00,
            "b12": 0x06,
            "b13": 0x60,
            "b14": self._get_extra_fan_settings(),
            "b15": 0x02,
            "b16": 0x80,
            "b17": 0x00,
            "b18": 0x06,

        }
        self._get_checksum(frame2_data)
        self._reverse(frame2_data)
        return bitstring.pack(fmt, **frame2_data).bin

    @staticmethod
    def _get_checksum(data):
        checksum = 0
        for k, b in data.items():
            checksum += b
        try:
            cs = bitstring.Bits(uint=checksum & 0xff, length=8)
        except bitstring.CreationError:
            from pprint import pformat
            logging.exception("Checksum value before reverse was {}".format(checksum))
            logging.exception("Checksum value before reverse after masking : {}".format(checksum & 0xff))
            logging.exception("data was\r\n{}".format(pformat(data)))
            sys.exit(0)
        checksum = {
            "cs": cs.uint
        }
        data.update(checksum)

    def _get_power_status_and_mode(self):
        return self.power.value + self.mode.value

    @staticmethod
    def _reverse(data):
        for k, b in data.items():
            b = bitstring.Bits(uint=b, length=8)
            ba = BitArray(b)
            ba.reverse()
            data[k] = ba.uint

    def _get_fan_settings(self):
        return self.fan_vertical_mode.value + self.fan_power.value

    def _get_extra_fan_settings(self):
        return self.fan_high_power.value + self.room_clean.value


def _test_panasonic(send_ir=False):
    from pap_logger import PaPLogger
    import sys
    import pigpio
    import AnaviInfraredPhat
    from time import sleep
    anavi_phat = None

    PaPLogger(level=logging.INFO, verbose_fmt=True)

    if send_ir:

        pi = pigpio.pi("192.168.104.218", 8888)
        if not pi.connected:
            logging.error("Could not connect to pigpiod on node1")
            sys.exit(0)

        anavi_phat = AnaviInfraredPhat.IRSEND(pi, r"/proc/cpuinfo")

    hvac = Panasonic(power=panasonic_enum.Power.OFF, temperature=21, mode=panasonic_enum.Mode.FAN,
                     fan_power=panasonic_enum.FanPower.FORCE3, fan_vertical_mode=panasonic_enum.FanVerticalMode.TOP)
    hvac.save_on_update = True
    from pprint import pformat
    logging.info(
        pformat(hvac.to_dict())
    )
    logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.bitstring))
    logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.wave))
    if send_ir:
        anavi_phat.send_ir(code=hvac.wave)
        sleep(5)
        sys.exit(0)


if __name__ == '__main__':
    _test_panasonic(send_ir=False)



#!/usr/bin/env python3
# coding=utf-8
"""
This class is a skeleton of all enumerations our parent Hvac class can access to.
All implementations of Hvac must overwrite Hvac class _enum property using their own enumerations
"""
from enum import Enum


class TempRange(Enum):
    MIN = 16
    STEP = 1.0
    MAX = 30


class Power(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = None


class Mode(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanVerticalMode(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanHorizontalMode(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class WideVanneMode(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class AreaMode(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanPower(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanHighPower(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanLong(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class RoomClean(Enum):
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


def get_enums_dict():
    """

    :return:
    """
    return {"Power": Power,
            "Mode": Mode,
            "FanVerticalMode": FanVerticalMode,
            "FanHorizontalMode": FanHorizontalMode,
            "WideVanneMode": WideVanneMode,
            "AreaMode": AreaMode,
            "FanPower": FanPower,
            "FanHighPower": FanHighPower,
            "FanLong": FanLong,
            "RoomClean": RoomClean,
            }



#!/usr/bin/env python3
# coding=utf-8
"""
Panasonic HVAC enumerations classes
Taken from https://qiita.com/awawaInu/items/25c6e17fcc2e655d5d42
also in english at https://www.analysir.com/blog/2014/12/27/reverse-engineering-panasonic-ac-infrared-protocol/
Half degrees aren't implemented.
"""
from enum import Enum


class TempRange(Enum):
    MIN = 16
    STEP = 1.0  # will need update later on
    MAX = 30


class Power(Enum):
    """
    Power
    """
    UNDEFINED = None
    OFF = 0b00001000
    ON = 0b00001001


class Mode(Enum):
    """
    Mode
    """
    FAN = 0b01100000
    DRY = 0b00100000
    COOL = 0b00110000
    HEAT = 0b01000000
    AUTO = 0b00000000
    UNDEFINED = DRY


class FanVerticalMode(Enum):
    """
    Fan vertical sweeping
    """
    TOP = 0b10000000
    MID_TOP = 0b01000000
    MIDDLE = 0b11000000
    MID_BOTTOM = 0b00100000
    BOTTOM = 0b10100000
    SWING = 0b11110000
    UNDEFINED = SWING


class FanHorizontalMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanPower(Enum):
    """
    Fan power
    """
    AUTO = 0b1100
    FORCE1 = 0b0010
    FORCE2 = 0b0101
    FORCE3 = 0b1010
    FORCE4 = 0b1110
    UNDEFINED = AUTO


class FanHighPower(Enum):
    """
    Extra power settings
    """
    QUIET = 0x04
    POWERFUL = 0x80
    NORMAL = 0
    UNDEFINED = NORMAL


class WideVanneMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class AreaMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanLong(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class RoomClean(Enum):
    ON = 0x02
    OFF = 0x00
    UNDEFINED = OFF


def get_enums_dict():
    """

    :return:
    """
    return {"Power": Power,
            "Mode": Mode,
            "FanVerticalMode": FanVerticalMode,
            "FanPower": FanPower,
            "FanHighPower": FanHighPower,
            "RoomClean": RoomClean,
            }


if __name__ == '__main__':
    print([FanVerticalMode])
    print(list(map(str, FanVerticalMode)))



#!/usr/bin/env python3
# coding=utf-8
"""
Toshiba HVAC enumerations classes
"""
from enum import Enum


class TempRange(Enum):
    MIN = 16
    STEP = 1.0
    MAX = 30


class Power(Enum):
    """
    Power
    """
    UNDEFINED = None
    OFF = 0x00  # TODO
    ON = 0x8f  # TODO


class Mode(Enum):
    """
    Mode
    """
    AUTO = 0x8
    DRY = 0x4
    COOL = 0x0
    HEAT = 0xc
    FAN = 0x1  # TODO
    UNDEFINED = AUTO


class FanVerticalMode(Enum):
    """
    Fan vertical sweeping
    """
    SWING = 0x0  # TODO
    NOSWING = 0x0  # TODO
    UNDEFINED = NOSWING


class FanPower(Enum):
    """
    Fan power
    """
    AUTO = 0x5  # TODO
    FORCE1 = 0xc  # TODO
    FORCE2 = 0x2  # TODO
    FORCE3 = 0xa  # TODO
    FORCE4 = 0x6  # TODO
    FORCE5 = 0xe  # TODO
    UNDEFINED = AUTO


class FanHorizontalMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class WideVanneMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class AreaMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanHighPower(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanLong(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class RoomClean(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


def get_enums_dict():
    """

    :return:
    """
    return {"Power": Power,
            "Mode": Mode,
            "FanVerticalMode": FanVerticalMode,
            "FanPower": FanPower,
            }


if __name__ == '__main__':
    print([FanVerticalMode])
    print(list(map(str, FanVerticalMode)))



#!/usr/bin/env python3
# coding=utf-8
"""
Daikin HVAC enumerations classes
"""
from enum import Enum


class TempRange(Enum):
    MIN = 16
    STEP = 1.0
    MAX = 30


class Power(Enum):
    """
    Daikin Power
    """
    UNDEFINED = None
    ON = 0x9
    OFF = 0x8


class Mode(Enum):
    """
    Daikin Mode
    """
    AUTO = 0x0
    DRY = 0x2
    COOL = 0x3
    HEAT = 0x4
    FAN = 0x6
    UNDEFINED = AUTO


class FanVerticalMode(Enum):
    """
    Daikin Fan vertical sweeping
    """
    SWING = 0xf
    BOTTOM = 0x50
    BOTTOM_MIDDLE = 0x40
    MIDDLE = 0x30
    TOP_MIDDLE = 0x20
    TOP = 0x10
    UNDEFINED = SWING


class FanHorizontalMode(Enum):
    """
    Daikin Fan horizontal sweeping
    """
    AREA_SWING_LEFT_CENTER = 1
    AREA_SWING_RIGHT_CENTER = 2
    AREA_SWING_FULL = 3
    AREA_KAZE_YOKE = 4
    UNDEFINED = None


class FanPower(Enum):
    """
    Daikin Fan power
    """
    AUTO = 0xa
    QUIET = 0xb
    FORCE1 = 0x3
    FORCE2 = 0x4
    FORCE3 = 0x5
    FORCE4 = 0x6
    FORCE5 = 0x7
    UNDEFINED = AUTO


class WideVanneMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class AreaMode(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanHighPower(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class FanLong(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


class RoomClean(Enum):
    """
    not available
    """
    NOT_AVAILABLE = -1
    UNDEFINED = NOT_AVAILABLE


def get_enums_dict():
    """

    :return:
    """
    return {"Power": Power,
            "Mode": Mode,
            "FanVerticalMode": FanVerticalMode,
            "FanHorizontalMode": FanHorizontalMode,
            "FanPower": FanPower,
            }


if __name__ == '__main__':
    print(Mode.UNDEFINED)
    print(Mode.AUTO)
